% ************************************************************************ %
%                                                                          %
%                 Diagnostic Tests for Homoskedasticity                    %
%              in Spatial Cross-Sectional or Panel Models                  %
%              Matlab code created by A. Pirotte & Z. Yang                 %
%                                                                          %
%        Spatial Dynamic Panel Data (SDPD) Model - September 2020          %
%                (Section 4 of the paper mentioned above)                  %
%                                                                          %
%                          Cigarette demand data                           %
%             of Baltagi and Levin (1986), Baltagi and Li (2004)           %
%                                                                          %
%************************************************************************* %

% No need to modify the environment initialization

clc; clear global; close all;
format short; echo off;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%% PART TO MODIFY %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% You have to change this part following the characteristics of your own
% data set

disp('              Cigarette demand data set                 ')           % Should be modified
disp(' of Baltagi and Levin (1986), Baltagi and Li (2004)     ')           % Should be modified
disp('   ')

% Data dimension characteristics

n       = 46                  ; % States (number of individuals)           % Should be modified
t       =  6                  ; % 1979-1984 (number of years, months...)   % Should be modified
nt      = n*t                 ; % Total number of observations

% Display of the dimensions of the data set and variable names
disp(['Number of US States (N)    ',num2str(n)])                           % Should be modified
disp(['Number of periods (T)      ',num2str(t),'   (1979-1982)'])          % Should be modified
disp(['Number of obs. (NT)        ',num2str(nt)])                          % Should be modified
disp(' ')                                                              
disp('List of the variables ')                                         
init = ['y  = ln(CIG)   - Real per capita sales of cigarettes          ';  % Should be modified
        'x1 = ln(P)     - Average retail price of a pack of cigarettes ';  % Should be modified
        'x2 = ln(I)     - Real per capita disposable Income            ';  % Should be modified
        'x3 = ln(Pop)   - Population                                   ';  % Should be modified  
        'x4 = ln(Pop16) - Population above 16                          ']; % Should be modified
disp(init)

% Spatial matrix - First order contiguity matrix

wb  = xlsread('Spat-Sym-US.xls')          ; % uploaded the spatial contiguity U.S. states matrix    % Should be modified
w   = wb./kron(ones(1,n),(ones(1,n)*wb')'); % Row-normalization
ev  = eig(w)                              ;
lb  = 1/min(ev) + 0.0001                  ;
ub  = 1/max(ev) - 0.0001                  ;
W1  = w                                   ;
W2  = w                                   ;
W3  = w                                   ;

%  Variables 

dcig   = xlsread('cigarette79_84.xls'); % uploaded the demand data set     % Should be modified
lcig   = dcig(:,3)     ;                                                   % Should be modified                      
lpx    = dcig(:,4)     ;                                                   % Should be modified
ly     = dcig(:,6)     ;                                                   % Should be modified
lpop   = log(dcig(:,7));                                                   % Should be modified
lpop16 = log(dcig(:,8));                                                   % Should be modified
st     = dcig(:,2)     ;                                                   % Should be modified

locig   = reshape(lcig,n,t)  ;                                             % Should be modified
lopx    = reshape(lpx,n,t)   ;                                             % Should be modified
loy     = reshape(ly,n,t)    ;                                             % Should be modified
lopop   = reshape(lpop,n,t)  ;                                             % Should be modified
lopop16 = reshape(lpop16,n,t);                                             % Should be modified
state   = reshape(st,n,t)    ;                                             % Should be modified

lcig    = locig(:,1:t)  ;                                                  % Should be modified
lpx     = lopx(:,1:t)   ;                                                  % Should be modified
ly      = loy(:,1:t)    ;                                                  % Should be modified
lpop16  = lopop16(:,1:t);                                                  % Should be modified

%%%%%%%%% 1979 is used as y_0 %%%%%%%%%%%%%%%%%%%%

%%%%%%%%% Individual means of Zn variables (Heteroskedascity variables)

i_XN1   = mean(lopx(:,2:t),2)       ; % Vector of individual means of x1   % Should be modified
i_XN2   = mean(loy(:,2:t),2)        ; % Vector of individual means of x2   % Should be modified
i_XN3   = mean(lopop16(:,2:t),2)    ; % Vector of individual means of x4   % Should be modified
ZNN     = cell(n,1)                 ;
ZNN{1}  = i_XN1                     ;                                      % Should be modified
ZNN{2}  = i_XN2                     ;                                      % Should be modified
ZNN{3}  = i_XN3                     ;                                      % Should be modified
ZN      = [kron(ones(t-1,1),ZNN{1}) kron(ones(t-1,1),ZNN{2}) kron(ones(t-1,1),ZNN{3})];   % Should be modified
[lz,cz] = size(ZN)                  ;

X_M     = [lpx(:) ly(:)]            ;
dx      = length(X_M(1,:))          ;

% First-difference of X

DxN1  = lpx(:,3:t) - lpx(:,2:t-1);                                         % Should be modified 
DxN2  = ly(:,3:t)  - ly(:,2:t-1) ;                                         % Should be modified  
DXN1  = DxN1(:)                  ;                                         % Should be modified
DXN2  = DxN2(:)                  ;                                         % Should be modified
DXN   = [DXN1 DXN2]              ;                                         % Should be modified

% First-difference of Y

DyN   = locig(:,3:t)    - locig(:,2:t-1)          ;                        % Should be modified
DyN_1 = locig(:,2:t-1)  - locig(:,1:t-2)          ;                        % Should be modified
DYN   = DyN(:)                                    ;                        % Should be modified
DYN_1 = DyN_1(:)                                  ;                        % Should be modified
XXC   = [DXN1 DXN2, DYN_1, kron(eye(t-2),w)*DYN_1];                        % Should be modified

disp(' ')
disp('   Dynamic Spatial Panel Data (DSPD) Model    ')
disp('______________________________________________')
disp(' ')
disp('Individual means of Zn variables : x1(i.) x2(i.) x4(i.)')            % Should be modified 

t     = t - 1       ;
In    = eye(n)      ;
It_1  = eye(t-1)    ;
e_n   = ones(n,1)   ;
e_T_1 = ones(t-1,1) ;

% Creating the matrix C0 for M-Estimator

C0    = Fn_CMat(t)  ;
CC    = kron(C0,In) ;
IC0   = pinv(C0)    ;
IC    = kron(IC0,In);
I0    = zeros(n,n)  ;
C_N_1 = IC          ;

% Heteroskedascity variable transformations

C_Zt_n    = cell(n*(t-1),1)       ;
C_Zt_n{1} = kron(IC0,diag(ZNN{1}));                                        % Should be modified
C_Zt_n{2} = kron(IC0,diag(ZNN{2}));                                        % Should be modified
C_Zt_n{3} = kron(IC0,diag(ZNN{3}));                                        % Should be modified                

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%% END OF MODIFICATIONS %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

nt  = n*(t-1)    ;
p   = size(DXN,2);  
df  = nt - p     ;      
fc  = df/nt      ;  

% Spatial matrices

W1N = kron(It_1,w);
W2N = kron(It_1,w);
W3N = kron(It_1,w);  

% M-Estimator 
    
start = [ .0   .0   .0   .0 ]                    ;
delta = FnAQSE_STLE(DYN,DYN_1,DXN,w,w,w,C0,start);
rhoh1M = delta(1);  lamh1M = delta(2); lamh2M = delta(3); lamh3M = delta(4); 
B1     = In - lamh1M*w                           ;
B2     = rhoh1M*In + lamh2M*w                    ;
B3     = In - lamh3M*w                           ;  
DDY    = kron(It_1,B1)*DYN - kron(It_1,B2)*DYN_1 ;
IOm    = kron(IC0,B3'*B3)                        ;
betahM = pinv(DXN'*IOm*DXN)*DXN'*IOm*DDY         ;
du     = DDY - DXN*betahM                        ;
sig2hM = du'*IOm*du/nt                           ;

B1nh   = B1                  ;   
B3nh   = B3                  ; 
iB1nh  = pinv(B1nh)          ; 
iB3nh  = pinv(B3nh)          ; 
B1N    = kron(It_1,B1nh)     ;
B3N    = kron(It_1,B3nh)     ;
iB1N   = kron(It_1,iB1nh)    ;
iB3N   = kron(It_1,iB3nh)    ;
G3     = w*iB3nh             ;
B3CB3  = B3N'*C_N_1*B3N      ;
BDY    = B1N*DYN             ;
B2nh   = In*rhoh1M + lamh2M*w;   
B2N    = kron(It_1,B2nh)     ;
BDYN   = BDY  - B2N*DYN_1    ;
res_M  = BDYN - DXN*betahM   ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%               Computation of the Statistic T^r_DSPD                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

eps   = 0.00001     ;  % Numerical derivatives
Bn    = iB1nh*B2nh  ; 
Bn2In = Bn - 2*In   ;
InBn  = (In - Bn)   ;
InBn2 = (In - Bn)^2 ;

dpar  = [rhoh1M, lamh1M, lamh2M]; 
D_N   = Fn_DMat(W1,W2,t,dpar)   ;
D_N_1 = Fn_DMat_1(W1,W2,t,dpar) ;
BB    = iB1nh*B2                ;

% Derivative of D_1 and D w.r.t. rho:
    if t == 2
       D_1r = I0     ;
       DLr  = iB1nh^2;
    elseif t == 3 
       D_1r = [I0, I0; iB1nh^2, I0]                           ;
       DLr  = [iB1nh^2 I0;(-2*iB1nh^2*(In-iB1nh*B2))  iB1nh^2];
    else  %using numerical derivatives
       dpar = [rhoh1M+eps,lamh1M,lamh2M]           ;
       DLr = (Fn_DMat(W1,W2,t,dpar)    - D_N)/eps  ;
       D_1r = (Fn_DMat_1(W1,W2,t,dpar) - D_N_1)/eps;
    end

    % Derivative of D_1 and D w.r.t. lambda1, lambda2:
    IWI1 = iB1nh*W1*iB1nh                                    ;
    IWI2 = iB1nh*W2*iB1nh                                    ;
    te1  = IWI1*B2*iB1nh+BB*IWI1-2*IWI1                      ;
    te2  = (iB1nh*W1*BB^2+BB*iB1nh*W1*BB-2*iB1nh*W1*BB)*iB1nh;
    te3  = (iB1nh*W2*BB+BB*iB1nh*W2-2*iB1nh*W2)*iB1nh        ; 
    if t == 2
        D_11 = IWI1; D_12 = I0            ;
        DL1  = te1 ; DL2  = iB1nh*W2*iB1nh;
    elseif t == 3    
        D_11 = [IWI1, I0; te1, IWI1]               ;
        D_12 = [I0, I0; IWI2, I0]                  ;                  
        DL1  = [te1, IWI1; te2+(In-BB)^2*IWI1, te1];
        DL2  = [IWI2, I0; te3, IWI2]               ;
    else  % using numerical derivatives
        dpar = [rhoh1M,lamh1M+eps,lamh2M]           ;
        DL1  = (Fn_DMat(W1,W2,t,dpar)   - D_N)/eps  ;
        D_11 = (Fn_DMat_1(W1,W2,t,dpar) - D_N_1)/eps;
    
        dpar = [rhoh1M,lamh1M,lamh2M+eps]           ;
        DL2  = (Fn_DMat(W1,W2,t,dpar)   - D_N)/eps  ;
        D_12 = (Fn_DMat_1(W1,W2,t,dpar) - D_N_1)/eps;
    end    
D_N_1Rho = D_1r; D_N_1L1h = D_11; D_N_1L2h = D_12;
D_NRho   = DLr ; D_NL1h   = DL1 ; D_NL2h   = DL2 ;

sig2i         = 1/sig2hM                                         ;
DUN           = B1N*DYN - B2N*DYN_1 - DXN*betahM                 ;
DVN           = B3N*DUN                                          ;

Sigma_N_bb    = -sig2i*DXN'*B3CB3*DXN                            ;
Sigma_N_bsig2 = -sig2i^2*DXN'*B3N'*C_N_1*DVN                     ;  
Sigma_N_brho  = -sig2i*DXN'*B3CB3*DYN_1                          ; 
Sigma_N_bl1   = -sig2i*DXN'*B3CB3*W1N*DYN                        ;
Sigma_N_bl2   = -sig2i*DXN'*B3CB3*W2N*DYN_1                      ;
Sigma_N_bl3   = -sig2i*DXN'*(W3N'*C_N_1*B3N + B3N'*C_N_1*W3N)*DUN;

Sigma_N_sig2b    = -sig2i^2*DXN'*B3N'*C_N_1*DVN           ; 
Sigma_N_sig2sig2 = -sig2i^3*DVN'*C_N_1*DVN + .5*nt*sig2i^2;   
Sigma_N_sig2rho  = -sig2i^2*DVN'*C_N_1*B3N*DYN_1          ;
Sigma_N_sig2l1   = -sig2i^2*DVN'*C_N_1*B3N*W1N*DYN        ; 
Sigma_N_sig2l2   = -sig2i^2*DVN'*C_N_1*B3N*W2N*DYN_1      ;            
Sigma_N_sig2l3   = -sig2i^2*DUN'*W3N'*C_N_1*B3N*DUN       ;

Sigma_N_rhob    = -sig2i*DXN'*B3CB3*DYN_1                               ; 
Sigma_N_rhosig2 = -sig2i^2*DVN'*C_N_1*B3N*DYN_1                         ; 
Sigma_N_rhorho  = -sig2i*DYN_1'*B3CB3*DYN_1      + trace(C_N_1*D_N_1Rho);
Sigma_N_rhol1   = -sig2i*DYN'*W1N'*B3CB3*DYN_1   + trace(C_N_1*D_N_1L1h);
Sigma_N_rhol2   = -sig2i*DYN_1'*W2N'*B3CB3*DYN_1 + trace(C_N_1*D_N_1L2h);
Sigma_N_rhol3   = -sig2i*DUN'*(W3N'*C_N_1*B3N + B3N'*C_N_1*W3N)*DYN_1   ;  

Sigma_N_l1b    = -sig2i*DXN'*B3CB3*W1N*DYN                                 ; 
Sigma_N_l1sig2 = -sig2i*DVN'*C_N_1*B3N*W1N*DYN                             ; 
Sigma_N_l1rho  = -sig2i*DYN_1'*B3CB3*W1N*DYN      + trace(C_N_1*D_NRho*W1N);
Sigma_N_l1l1   = -sig2i*DYN'*W1N'*B3CB3*W1N*DYN   + trace(C_N_1*D_NL1h*W1N);
Sigma_N_l1l2   = -sig2i*DYN_1'*W2N'*B3CB3*W1N*DYN + trace(C_N_1*D_NL2h*W1N);
Sigma_N_l1l3   = -sig2i*DUN'*(W3N'*C_N_1*B3N + B3N'*C_N_1*W3N)*W1N*DYN     ; 

Sigma_N_l2b    = -sig2i*DXN'*B3CB3*W2N*DYN_1                                            ; 
Sigma_N_l2sig2 = -sig2i^2*DVN'*C_N_1*B3N*W2N*DYN_1                                      ; 
Sigma_N_l2rho  = -sig2i*DYN_1'*B3CB3*W2N*DYN_1      + trace(C_N_1*D_N_1Rho*W2N)         ;
Sigma_N_l2l1   = -sig2i*DYN'*W1N'*B3N'*C_N_1*B3N*W2N*DYN_1   + trace(C_N_1*D_N_1L1h*W2N);
Sigma_N_l2l2   = -sig2i*DYN_1'*W2N'*B3CB3*W2N*DYN_1 + trace(C_N_1*D_N_1L2h*W2N)         ;
Sigma_N_l2l3   = -sig2i*DUN'*(W3N'*C_N_1*B3N + B3N'*C_N_1*W3N)*W2N*DYN_1                ; 

CA3 = kron(IC0,(W3'*B3nh+B3nh'*W3));

Sigma_N_l3b    = -.5*sig2i*(DXN'*CA3*DUN           + (DUN'*CA3*DXN)')          ;              
Sigma_N_l3sig2 = -.5*sig2i^2*DUN'*CA3*DUN                                      ; 
Sigma_N_l3rho  = -.5*sig2i*(DYN_1'*CA3*DUN         + DUN'*CA3*DYN_1)           ;
Sigma_N_l3l1   = -.5*sig2i*(DYN'*W1N'*CA3*DUN      + DUN'*CA3*W1N*DYN)         ;
Sigma_N_l3l2   = -sig2i*DUN'*(B3N'*C_N_1*W3N + W3N'*C_N_1*B3N)*W2N*DYN_1       ;  
Sigma_N_l3l3   = -sig2i*(DUN'*kron(IC0,W3'*W3)*DUN) - (t-1)*trace((W3*iB3nh)^2); 

SIGMA_N_thth  = -[Sigma_N_bb     Sigma_N_bsig2    Sigma_N_brho    Sigma_N_bl1    Sigma_N_bl2    Sigma_N_bl3   ;...
                  Sigma_N_sig2b'  Sigma_N_sig2sig2 Sigma_N_sig2rho Sigma_N_sig2l1 Sigma_N_sig2l2 Sigma_N_sig2l3;...
                  Sigma_N_rhob'   Sigma_N_rhosig2  Sigma_N_rhorho  Sigma_N_rhol1  Sigma_N_rhol2  Sigma_N_rhol3 ;...
                  Sigma_N_l1b'    Sigma_N_l1sig2   Sigma_N_l1rho   Sigma_N_l1l1   Sigma_N_l1l2   Sigma_N_l1l3  ;...
                  Sigma_N_l2b'    Sigma_N_l2sig2   Sigma_N_l2rho   Sigma_N_l2l1   Sigma_N_l2l2   Sigma_N_l2l3  ;...
                  Sigma_N_l3b'    Sigma_N_l3sig2   Sigma_N_l3rho   Sigma_N_l3l1   Sigma_N_l3l2   Sigma_N_l3l3];

Sigma_N_alb    = cell(1,dx);
Sigma_N_alsig2 = cell(1,1) ;
Sigma_N_alrho  = cell(1,1) ;
Sigma_N_all1   = cell(1,1) ;
Sigma_N_all2   = cell(1,1) ;
Sigma_N_all3   = cell(1,1) ;
for i = 1:cz  
  Sigma_N_alb{i}    = -.5*sig2i*((DXN'*B3N'*C_Zt_n{i}*DVN)' + DVN'*C_Zt_n{i}*B3N*DXN); 
  Sigma_N_alsig2{i} = -.5*sig2i^2*DVN'*C_Zt_n{i}*DVN                                 ; 
  Sigma_N_alrho{i}  = -sig2i*DYN_1'*B3N'*C_Zt_n{i}*DVN                               ;
  Sigma_N_all1{i}   = -sig2i*DYN'*W1N'*B3N'*C_Zt_n{i}*DVN                            ;
  Sigma_N_all2{i}   = -sig2i*DYN_1'*W2N'*B3N'*C_Zt_n{i}*DVN                          ;  
  Sigma_N_all3{i}   = -sig2i*DUN'*B3N'*C_Zt_n{i}*W3N*DUN                             ; 
end

mat1 = [];
mat2 = [];
mat3 = [];
mat4 = [];
mat5 = [];
mat6 = [];
for i = 1:cz
  mat1 = [mat1 ; Sigma_N_alb{i}   ];
  mat2 = [mat2 ; Sigma_N_alsig2{i}];
  mat3 = [mat3 ; Sigma_N_alrho{i}] ;
  mat4 = [mat4 ; Sigma_N_all1{i}]  ;
  mat5 = [mat5 ; Sigma_N_all2{i}]  ;
  mat6 = [mat6 ; Sigma_N_all3{i}]  ;
end
              
SIGMA_N_alth  = -[mat1 mat2 mat3 mat4 mat5 mat6];
GAMMA_tilde_N = SIGMA_N_alth*pinv(SIGMA_N_thth) ;

%%%%% B, B_1, R and R_1 %%%%%%

Bb   = zeros(nt,nt);
Bb_1 = zeros(nt,nt);
for i = 1:(t-1)
 i0 = n*(i-1)+1; i1 = n*i;  
 Bb(i0:i1,i0:i1) = In;
  for j = (i+1):(t-1)
    i00 = n*(j-1)+1; i11 = n*j;
    Bb(i00:i11,i0:i1)   = Bn^(j-i)  ;
    Bb_1(i00:i11,i0:i1) = Bn^(j-i-1);
  end
end
        
Rb   = Bn;
Rb_1 = In;
 for i = 2:(t-1)
    Rb   = blkdiag(Rb,Bn^i)      ;
    Rb_1 = blkdiag(Rb_1,Bn^(i-1));
 end               

%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
ETA   = Bb*iB1N*DXN*betahM        ;
ETA_1 = Bb_1*iB1N*DXN*betahM      ;
Ss    = Bb*iB1N*iB3N              ;
Ss_1  = Bb_1*iB1N*iB3N            ;

Pi1  = sig2i*C_N_1*B3N*DXN      ;
Pi2  = sig2i*C_N_1*B3N*ETA_1    ;
Pi3  = sig2i*C_N_1*B3N*W1N*ETA  ;
Pi4  = sig2i*C_N_1*B3N*W2N*ETA_1;

Phi1 = .5*sig2i^2*C_N_1                        ;
Phi2 = sig2i*C_N_1*B3N*Ss_1                    ;
Phi3 = sig2i*C_N_1*B3N*W1N*Ss                  ;
Phi4 = sig2i*C_N_1*B3N*W2N*Ss_1                ;
Phi5 = .5*sig2i*kron(IC0,W3*iB3nh + iB3nh'*W3');

Phi6 = cell(nt,nt);
for i = 1:cz  
  Phi6{i} = .5*sig2i*C_Zt_n{i}; 
end

Psi1 = sig2i*C_N_1*B3N*Rb_1    ;
Psi2 = sig2i*C_N_1*B3N*W1N*Rb  ;
Psi3 = sig2i*C_N_1*B3N*W2N*Rb_1;   

du    = res_M       ;
It    = It_1        ;
B1    = B1nh        ;
B3    = B3nh        ;
IB1   = iB1nh       ;
IB3   = iB3nh       ;
Dy1   = DYN_1(1:n,1);
sigh1 = sqrt(sig2hM);

dv   = kron(It,B3)*du ;
dvv  = reshape(dv,n,t-1);
 
    dxi1 = zeros(n,t-1);
    dxi2 = zeros(n,t-1);
    dxi3 = zeros(n,t-1);
    dxi4 = zeros(n,t-1);
    dxi5 = zeros(n,t-1);
    dxi6 = zeros(n,t-1,cz);
    for it = 1:(t-1)
      it1  = (it-1)*n+1; it2 = it*n;  
      for is = 1:(t-1)
        is1 = (is-1)*n+1; is2 = is*n;
        P1ts = Phi1(it1:it2, is1:is2);     P1st = Phi1(is1:is2, it1:it2);
        P2ts = Phi2(it1:it2, is1:is2);     P2st = Phi2(is1:is2, it1:it2);
        P3ts = Phi3(it1:it2, is1:is2);     P3st = Phi3(is1:is2, it1:it2);
        P4ts = Phi4(it1:it2, is1:is2);     P4st = Phi4(is1:is2, it1:it2);
        P5ts = Phi5(it1:it2, is1:is2);     P5st = Phi5(is1:is2, it1:it2);
        P6ts = cell(n,n);
        P6st = cell(n,n);
        for i = 1:cz  
         P6ts{i} = Phi6{i}(it1:it2,is1:is2); P6st{i} = Phi6{i}(is1:is2,it1:it2);
        end
        dvvs = dvv(:,is);
        dxi1(:,it) = dxi1(:,it)+(triu(P1st)'+tril(P1ts)-diag(diag(P1ts)))*dvvs; 
        dxi2(:,it) = dxi2(:,it)+(triu(P2st)'+tril(P2ts)-diag(diag(P2ts)))*dvvs; 
        dxi3(:,it) = dxi3(:,it)+(triu(P3st)'+tril(P3ts)-diag(diag(P3ts)))*dvvs; 
        dxi4(:,it) = dxi4(:,it)+(triu(P4st)'+tril(P4ts)-diag(diag(P4ts)))*dvvs; 
        dxi5(:,it) = dxi5(:,it)+(triu(P5st)'+tril(P5ts)-diag(diag(P5ts)))*dvvs; 
        for i = 1:cz 
          dxi6(:,it,i) = dxi6(:,it,i)+(triu(P6st{i})'+tril(P6ts{i})-diag(diag(P6ts{i})))*dvvs; 
        end
      end  
    end
      
    Pta1 = zeros(n,n);
    Pta2 = zeros(n,n);
    Pta3 = zeros(n,n);
    for is = 1:(t-1)
      is1  = (is-1)*n+1; is2 = is*n;
      Pta1 = Pta1 + Psi1(1:n, is1:is2);
      Pta2 = Pta2 + Psi2(1:n, is1:is2);
      Pta3 = Pta3 + Psi3(1:n, is1:is2);
    end
    Thet1 = Pta1*IB1*IB3;
    Thet2 = Pta2*IB1*IB3;
    Thet3 = Pta3*IB1*IB3;

    gr1c = zeros(n,1);
    g11c = zeros(n,1);
    g21c = zeros(n,1);
    for it = 2:(t-1)
      Pta1 = zeros(n,n);
      Pta2 = zeros(n,n);
      Pta3 = zeros(n,n);
      it1  = (it-1)*n+1; it2 = it*n;  
      for is = 1:(t-1)
        is1  = (is-1)*n+1; is2 = is*n;
        Pta1 = Pta1 + Psi1(it1:it2, is1:is2);
        Pta2 = Pta2 + Psi2(it1:it2, is1:is2);
        Pta3 = Pta3 + Psi3(it1:it2, is1:is2);
      end 
      gr1c = gr1c + dvv(:,it).*(Pta1*Dy1);
      g11c = g11c + dvv(:,it).*(Pta2*Dy1);
      g21c = g21c + dvv(:,it).*(Pta3*Dy1);
    end

gb = cell(n,1);
for i = 1:dx
  gb{i} = sum(reshape(Pi1(:,i),n,t-1).*dvv, 2);
end
gs  = sum(dvv.*dxi1,2)-(t-1)/(2*sigh1^2)  ; 
gl3 = sum(dvv.*dxi5,2)-(t-1)*diag(G3)     ;
    
glalpha = cell(n,1);
for i = 1:cz    
 glalpha{i} = sum(dvv.*dxi6(:,:,i),2)-.5*(t-1)*ZNN{i}.*ones(n,1);  % times .5
end

Dy1o = B3*B1*Dy1                                   ;
dvv1 = dvv(:,1)                                    ;
dr   = sigh1^2*sum(reshape(diag(CC*Phi2),n,t-1), 2);
dl1  = sigh1^2*sum(reshape(diag(CC*Phi3),n,t-1), 2);
dl2  = sigh1^2*sum(reshape(diag(CC*Phi4),n,t-1), 2);

tem  = triu(Thet1)'+tril(Thet1)-2*diag(diag(Thet1));
gr1a = dvv1.*(tem*Dy1o)                            ;
gr1b = diag(Thet1).*(dvv1.*Dy1o+sigh1^2)           ;
gr2  = sum(reshape(Pi2,n,t-1).*dvv, 2)             ;
gr3  = sum(dvv.*dxi2, 2) - dr                      ;  
gr   = gr1a + gr1b + gr1c + gr2 + gr3              ;

tem  = triu(Thet2)'+tril(Thet2)-2*diag(diag(Thet2));
g11a = dvv1.*(tem*Dy1o)                            ;
g11b = diag(Thet2).*(dvv1.*Dy1o+sigh1^2)           ; 
g12  = sum(reshape(Pi3,n,t-1).*dvv, 2)             ;
g13  = sum(dvv.*dxi3, 2) - dl1                     ; 
gl1  = g11a + g11b + g11c + g12 + g13              ;

tem = triu(Thet3)'+tril(Thet3)-2*diag(diag(Thet3));
g21a= dvv1.*(tem*Dy1o)                            ;
g21b= diag(Thet3).*(dvv1.*Dy1o+sigh1^2)           ;
g22 = sum(reshape(Pi4,n,t-1).*dvv, 2)             ;
g23 = sum(dvv.*dxi4, 2) - dl2                     ;  
gl2 = g21a + g21b + g21c + g22 + g23              ;


g_thetab = zeros(n,dx);
for i = 1:dx
  g_thetab(:,i) = gb{i};
end        
g_ni_theta = [g_thetab, gs, gr, gl1, gl2, gl3];

g_ni_alpha = zeros(n,cz);
for i = 1:cz
  g_ni_alpha(:,i) = glalpha{i};
end                                                                       

sg_ni_alpha = sum(g_ni_alpha)                            ; 
MatGG       = g_ni_alpha - g_ni_theta*GAMMA_tilde_N'     ;
T_r_DSPD    = sg_ni_alpha*pinv(MatGG'*MatGG)*sg_ni_alpha';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%            Computation of the Statistic T^r*_DSPD                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

start1  = [0 0 0 0];
delta  = FnMAQSE_STLE(DYN,DYN_1,DXN,W1,W2,W3,C0,df,start1);
rhoh1M = delta(1);  lamh1M = delta(2); lamh2M = delta(3); lamh3M = delta(4);  

B1nh   = In - lamh1M*W1        ;   
B2nh   = In*rhoh1M + lamh2M*W2 ;   
B3nh   = In - lamh3M*W3        ; 
iB1nh  = pinv(B1nh)            ; 
i2B1nh = B1nh^(-2)             ;
iB3nh  = pinv(B3nh)            ; 
B1N    = kron(It_1,B1nh)       ;
B2N    = kron(It-1,B2nh)       ;
B3N    = kron(It_1,B3nh)       ;
iB1N   = kron(It_1,iB1nh)      ;
iB3N   = kron(It_1,iB3nh)      ;
i2B1N  = kron(It_1,i2B1nh)     ;
G3     = W3*iB3nh              ;
B3CB3  = B3N'*C_N_1*B3N        ;

IOm      = kron(IC0,B3nh'*B3nh)            ;
BDYN     = B1N*DYN - B2N*DYN_1             ;
betahM   = pinv(DXN'*IOm*DXN)*DXN'*IOm*BDYN;
res_M    = BDYN - DXN*betahM               ;
sig2hM   = res_M'*IOm*res_M/nt             ;

Ci05     = CC^(-0.5)                          ; 
G3np     = G3 + G3'                           ;
CA3b     = -CA3                               ;
CA3i     = kron(IC0,G3np)                     ;
CA3ib    = kron(C0,G3np)                      ;
ddCA3    = -DXN'*CA3b*DXN*(DXN'*IOm*DXN)^(-2) ;
DXIOXD   = pinv(DXN'*IOm*DXN)                 ;
PDOX     = DXN*DXIOXD*DXN'                    ;
PDOXO    = PDOX*IOm                           ;

DXBN     =  Ci05*B3N*DXN              ;
DXBNb    = -Ci05*W3N*DXN              ;
M_N      = eye(nt) - DXBN*DXIOXD*DXBN';
M_o      = Ci05*M_N*Ci05              ;
MoB3N    = M_o*B3N                    ;
sig2hM_s = (1/df)*DVN'*M_o*DVN        ;

mu_s_Rho  = - trace(MoB3N*D_N_1*iB3N)               ;
mu_s_l1   = - trace(MoB3N*W1N*D_N*iB3N)             ;
mu_s_l2   = - trace(MoB3N*W2N*D_N_1*iB3N)           ;
mu_s_l3   =  .5*trace(M_N*kron(It_1,G3np))          ;
mu_s_al   = cell(1,1);
for i = 1:cz
  mu_s_al{i} = .5*trace(M_N*kron(It_1,diag(ZNN{i})));   
end

Pi1s  = MoB3N*ETA_1    ;
Pi2s  = MoB3N*W1N*ETA  ;
Pi3s  = MoB3N*W2N*ETA_1;

Phi1s = MoB3N*Ss_1       - (1/df)*mu_s_Rho*M_o ;
Phi2s = MoB3N*W1N*Ss     - (1/df)*mu_s_l1*M_o  ;
Phi3s = MoB3N*W2N*Ss_1   - (1/df)*mu_s_l2*M_o  ;
Phi4s = .5*M_o*CA3ib*M_o - (1/df)*mu_s_l3*M_o  ;
Phi5s = cell(nt,nt);
for i = 1:cz
  Phi5s{i} = .5*M_o*C_Zt_n{i}*M_o - (1/df)*mu_s_al{i}*M_o;
end

Psi1s = MoB3N*Rb_1    ;  
Psi2s = MoB3N*W1N*Rb  ; 
Psi3s = MoB3N*W2N*Rb_1; 

dM_l3   = -((DXBNb*DXIOXD*DXBN') + (DXBN*ddCA3*DXBN') + (DXBN*DXIOXD*DXBNb'))        ;
dDVN_l3 = - W3N*DUN - B3N*(DXN*ddCA3*DXN'*IOm*BDYN) - B3N*(DXN*DXIOXD*DXN'*CA3b*BDYN);

Sigma_Ns_rhorhob =  - (DYN_1 - PDOXO*DYN_1)'*B3CB3*DYN_1...
                    - (-trace(MoB3N*D_N_1Rho*iB3N)*sig2hM_s - (2/df)*mu_s_Rho*(DYN_1 - PDOXO*DYN_1)'*MoB3N'*DVN);
Sigma_Ns_rhol1b  =  - (W1N*DYN - PDOXO*W1N*DYN)'*B3CB3*DYN_1... 
                    - (-trace(MoB3N*D_N_1L1h*iB3N)*sig2hM_s - (2/df)*mu_s_Rho*(W1N*DYN - PDOXO*W1N*DYN)'*MoB3N'*DVN);                             
Sigma_Ns_rhol2b  =  - (W2N*DYN_1 - PDOXO*W2N*DYN_1)'*B3CB3*DYN_1... 
                    - (-trace(MoB3N*D_N_1L2h*iB3N)*sig2hM_s - (2/df)*mu_s_Rho*(W2N*DYN_1 - PDOXO*W2N*DYN_1)'*MoB3N'*DVN);                
Sigma_Ns_rhol3b  =    dDVN_l3'*C_N_1*B3N*DYN_1 - DVN'*C_N_1*W3N*DYN_1...
                    - (-trace(Ci05*dM_l3*Ci05*B3N*D_N_1*iB3N - M_o*W3N*D_N_1*iB3N + MoB3N*D_N_1*W3N*B3N^(-2))*sig2hM_s + (2/df)*mu_s_Rho*dDVN_l3'*M_o*DVN...
                      + (1/df)*mu_s_Rho*DVN'*Ci05*dM_l3*Ci05*DVN);
                
Sigma_Ns_l1rhob  = - (DYN_1 - PDOXO*DYN_1)'*B3CB3*W1N*DYN...
                   - (-trace(MoB3N*W1N*D_NRho*iB3N)*sig2hM_s - (2/df)*mu_s_l1*(DYN_1 - PDOXO*DYN_1)'*MoB3N'*DVN);
Sigma_Ns_l1l1b   = - (W1N*DYN - PDOXO*W1N*DYN)'*B3CB3*W1N*DYN...
                   - (-trace(MoB3N*W1N*D_NL1h*iB3N)*sig2hM_s - (2/df)*mu_s_l1*(W1N*DYN - PDOXO*W1N*DYN)'*MoB3N'*DVN);
Sigma_Ns_l1l2b   = - (W2N*DYN_1 - PDOXO*W2N*DYN_1)'*B3CB3*W1N*DYN...
                   - (-trace(MoB3N*W1N*D_NL2h*iB3N)*sig2hM_s - (2/df)*mu_s_l1*(W2N*DYN_1 - PDOXO*W2N*DYN_1)'*MoB3N'*DVN);
Sigma_Ns_l1l3b   =    dDVN_l3'*C_N_1*B3N*W1N*DYN - DVN'*C_N_1*W3N*W1N*DYN...
                      - (-trace(Ci05*dM_l3*Ci05*B3N*W1N*D_N*iB3N - M_o*W3N*W1N*D_N*iB3N + MoB3N*W1N*D_N*W3N*B3N^(-2))*sig2hM_s + (2/df)*mu_s_l1*dDVN_l3'*M_o*DVN...
                      + (1/df)*mu_s_l1*DVN'*Ci05*dM_l3*Ci05*DVN);

Sigma_Ns_l2rhob  = - (DYN_1 - PDOXO*DYN_1)'*B3CB3*W2N*DYN_1...
                   - (-trace(MoB3N*W2N*D_N_1Rho*iB3N)*sig2hM_s - (2/df)*mu_s_l2*(DYN_1 - PDOXO*DYN_1)'*MoB3N'*DVN);   
Sigma_Ns_l2l1b   = - (W1N*DYN - PDOXO*W1N*DYN)'*B3CB3*W2N*DYN_1...
                   - (-trace(MoB3N*W2N*D_N_1L1h*iB3N)*sig2hM_s - (2/df)*mu_s_l2*(W1N*DYN - PDOXO*W1N*DYN)'*MoB3N'*DVN);               
Sigma_Ns_l2l2b   = - (W2N*DYN_1 - PDOXO*W2N*DYN_1)'*B3CB3*W2N*DYN_1...
                   - (-trace(MoB3N*W2N*D_N_1L2h*iB3N)*sig2hM_s - (2/df)*mu_s_l2*(W2N*DYN_1 - PDOXO*W2N*DYN_1)'*MoB3N'*DVN); 
Sigma_Ns_l2l3b   =    dDVN_l3'*C_N_1*B3N*W2N*DYN_1 - DVN'*C_N_1*W3N*W2N*DYN_1...
                      - (-trace(Ci05*dM_l3*Ci05*B3N*W2N*D_N_1*iB3N - M_o*W3N*W2N*D_N_1*iB3N + MoB3N*W2N*D_N_1*W3N*B3N^(-2))*sig2hM_s + (2/df)*mu_s_l2*dDVN_l3'*M_o*DVN...
                      + (1/df)*mu_s_l2*DVN'*Ci05*dM_l3*Ci05*DVN);
             
Sigma_Ns_l3rhob  = - (DYN_1 - PDOXO*DYN_1)'*B3N'*CA3i*DVN         - (-(2/df)*mu_s_l3*(DYN_1 - PDOXO*DYN_1)'*MoB3N'*DVN)        ;
Sigma_Ns_l3l1b   = - (W1N*DYN - PDOXO*W1N*DYN)'*B3N'*CA3i*DVN     - (-(2/df)*mu_s_l3*(W1N*DYN - PDOXO*W1N*DYN)'*MoB3N'*DVN)    ;              
Sigma_Ns_l3l2b   = - (W2N*DYN_1 - PDOXO*W2N*DYN_1)'*B3N'*CA3i*DVN - (-(2/df)*mu_s_l3*(W2N*DYN_1 - PDOXO*W2N*DYN_1)'*MoB3N'*DVN);  
Sigma_Ns_l3l3b   =    dDVN_l3'*CA3i*DVN + .5*DVN'*kron(IC0,G3^2+(G3^2)')*DVN...
                   - (.5*trace(dM_l3*kron(It_1,G3np) + M_N*kron(It_1,G3^2+(G3^2)'))*sig2hM_s + (2/df)*mu_s_l3*dDVN_l3'*M_o*DVN...
                      + (1/df)*mu_s_l3*DVN'*Ci05*dM_l3*Ci05*DVN);   

SIGMA_Ns_deldelb = -[Sigma_Ns_rhorhob Sigma_Ns_rhol1b Sigma_Ns_rhol2b Sigma_Ns_rhol3b;...
                     Sigma_Ns_l1rhob  Sigma_Ns_l1l1b  Sigma_Ns_l1l2b  Sigma_Ns_l1l3b;...
                     Sigma_Ns_l2rhob  Sigma_Ns_l2l1b  Sigma_Ns_l2l2b  Sigma_Ns_l2l3b;...
                     Sigma_Ns_l3rhob  Sigma_Ns_l3l1b  Sigma_Ns_l3l2b  Sigma_Ns_l3l3b];   
                 
Sigma_Ns_alrhob = cell(1,1);
Sigma_Ns_all1b  = cell(1,1) ;
Sigma_Ns_all2b  = cell(1,1) ;
Sigma_Ns_al3b   = cell(1,1) ;
for i = 1:cz  
  Sigma_Ns_alrhob{i} = - (DYN_1 - PDOXO*DYN_1)'*B3N'*C_Zt_n{i}*DVN         - (-(2/df)*mu_s_al{i}*(DYN_1 - PDOXO*DYN_1)'*MoB3N'*DVN); 
  Sigma_Ns_all1b{i}  = - (W1N*DYN - PDOXO*W1N*DYN)'*B3N'*C_Zt_n{i}*DVN     - (-(2/df)*mu_s_al{i}*(W1N*DYN - PDOXO*W1N*DYN)'*MoB3N'*DVN); 
  Sigma_Ns_all2b{i}  = - (W2N*DYN_1 - PDOXO*W2N*DYN_1)'*B3N'*C_Zt_n{i}*DVN - (-(2/df)*mu_s_al{i}*(W2N*DYN_1 - PDOXO*W2N*DYN_1)'*MoB3N'*DVN);
  Sigma_Ns_al3b{i}   =  dDVN_l3'*C_Zt_n{i}*DVN - (.5*trace(dM_l3*kron(It_1,diag(ZNN{i})))*sig2hM_s + (2/df)*mu_s_al{i}*dDVN_l3'*M_o*DVN...
                        + (1/df)*mu_s_al{i}*DVN'*Ci05*dM_l3*Ci05*DVN);                  
end
     
mat1 = [];
mat2 = [];
mat3 = [];
mat4 = [];
for i = 1:cz
  mat1 = [mat1 ; Sigma_Ns_alrhob{i}];
  mat2 = [mat2 ; Sigma_Ns_all1b{i}] ;
  mat3 = [mat3 ; Sigma_Ns_all2b{i}] ;
  mat4 = [mat4 ; Sigma_Ns_al3b{i}]  ;
end
                  
SIGMA_Ns_aldelb = -[mat1 mat2 mat3 mat4]                ;
GAMMA_tilde_Ns  = SIGMA_Ns_aldelb*pinv(SIGMA_Ns_deldelb);

    dxi1 = zeros(n,t-1)   ;
    dxi2 = zeros(n,t-1)   ;
    dxi3 = zeros(n,t-1)   ;
    dxi4 = zeros(n,t-1)   ;
    dxi5 = zeros(n,t-1,cz);    
    for it = 1:(t-1)
      it1  = (it-1)*n+1; it2 = it*n;  
      for is = 1:(t-1)
        is1 = (is-1)*n+1; is2 = is*n;
        P1ts = Phi1s(it1:it2,is1:is2); P1st = Phi1s(is1:is2,it1:it2);
        P2ts = Phi2s(it1:it2,is1:is2); P2st = Phi2s(is1:is2,it1:it2);
        P3ts = Phi3s(it1:it2,is1:is2); P3st = Phi3s(is1:is2,it1:it2);
        P4ts = Phi4s(it1:it2,is1:is2); P4st = Phi4s(is1:is2,it1:it2);
        P5ts = cell(n,n);
        P5st = cell(n,n);
        for i = 1:cz  
         P5ts{i} = Phi5s{i}(it1:it2,is1:is2); P5st{i} = Phi5s{i}(is1:is2,it1:it2);
        end
        dvvs = dvv(:,is);
        dxi1(:,it) = dxi1(:,it) + (triu(P1st,1)' + tril(P1ts,-1) + diag(diag(P1ts)))*dvvs; 
        dxi2(:,it) = dxi2(:,it) + (triu(P2st,1)' + tril(P2ts,-1) + diag(diag(P2ts)))*dvvs; 
        dxi3(:,it) = dxi3(:,it) + (triu(P3st,1)' + tril(P3ts,-1) + diag(diag(P3ts)))*dvvs; 
        dxi4(:,it) = dxi4(:,it) + (triu(P4st,1)' + tril(P4ts,-1) + diag(diag(P4ts)))*dvvs; 
        for i = 1:cz 
          dxi5(:,it,i) = dxi5(:,it,i) + (triu(P5st{i},1)' + tril(P5ts{i},-1) + diag(diag(P5ts{i})))*dvvs; 
        end
      end
    end
             
    Pta1 = zeros(n,n);
    Pta2 = zeros(n,n);
    Pta3 = zeros(n,n);
    for is = 1:(t-1)
      is1  = (is-1)*n+1; is2 = is*n;
      Pta1 = Pta1 + Psi1s(1:n,is1:is2);
      Pta2 = Pta2 + Psi2s(1:n,is1:is2);
      Pta3 = Pta3 + Psi3s(1:n,is1:is2);
    end
    Thet1 = Pta1*IB1*IB3;
    Thet2 = Pta2*IB1*IB3;
    Thet3 = Pta3*IB1*IB3;

    gr1c = zeros(n,1);
    g11c = zeros(n,1);
    g21c = zeros(n,1);
    for it = 2:(t-1)
      Pta1 = zeros(n,n);
      Pta2 = zeros(n,n);
      Pta3 = zeros(n,n);
      it1  = (it-1)*n+1; it2 = it*n;  
      for is = 1:(t-1)
        is1  = (is-1)*n+1; is2 = is*n;
        Pta1 = Pta1 + Psi1s(it1:it2,is1:is2);
        Pta2 = Pta2 + Psi2s(it1:it2,is1:is2);
        Pta3 = Pta3 + Psi3s(it1:it2,is1:is2);
      end 
      gr1c = gr1c + dvv(:,it).*(Pta1*Dy1);
      g11c = g11c + dvv(:,it).*(Pta2*Dy1);
      g21c = g21c + dvv(:,it).*(Pta3*Dy1);
    end
    
dr   = sig2hM_s*sum(reshape(diag(CC*Phi1s),n,t-1),2);
dl1  = sig2hM_s*sum(reshape(diag(CC*Phi2s),n,t-1),2);
dl2  = sig2hM_s*sum(reshape(diag(CC*Phi3s),n,t-1),2);
dl3  = sig2hM_s*sum(reshape(diag(CC*Phi4s),n,t-1),2);
dla  = cell(n,1);
for i = 1:cz
 dla{i} = sig2hM_s*sum(reshape(diag(CC*Phi5s{i}),n,t-1),2);    
end

gl3      = sum(dvv.*dxi4,2) - dl3 ;
glalpha  = cell(n,1);
for i = 1:cz
 glalpha{i} = sum(dvv.*dxi5(:,:,i),2) - dla{i};    
end
    
tem  = triu(Thet1,1)' + tril(Thet1,-1);
gr1a = dvv1.*(tem*Dy1o);
gr1b = diag(Thet1).*(dvv1.*Dy1o + sig2hM_s);
gr2  = sum(reshape(Pi1s,n,t-1).*dvv,2);
gr3  = sum(dvv.*dxi1,2) - dr; 
gr   = gr1a + gr1b + gr1c + gr2 + gr3;

tem  = triu(Thet2,1)' + tril(Thet2,-1);
g11a = dvv1.*(tem*Dy1o);
g11b = diag(Thet2).*(dvv1.*Dy1o + sig2hM_s);
g12  = sum(reshape(Pi2s,n,t-1).*dvv,2);
g13  = sum(dvv.*dxi2,2) - dl1; 
gl1  = g11a + g11b + g11c + g12 + g13;

tem  = triu(Thet3,1)' + tril(Thet3,-1);
g21a = dvv1.*(tem*Dy1o);
g21b = diag(Thet3).*(dvv1.*Dy1o + sig2hM_s);
g22  = sum(reshape(Pi3s,n,t-1).*dvv,2);
g23  = sum(dvv.*dxi3,2) - dl2; 
gl2  = g21a + g21b + g21c + g22 + g23;
    
g_ni_delta = [gr, gl1, gl2, gl3];
g_ni_alpha = zeros(n,cz)        ;
for i = 1:cz
  g_ni_alpha(:,i) = glalpha{i};
end     
sg_ni_alpha = sum(g_ni_alpha) ;

MatGG     = g_ni_alpha - g_ni_delta*GAMMA_tilde_Ns'    ;
T_rs_DSPD = sg_ni_alpha*pinv(MatGG'*MatGG)*sg_ni_alpha';

disp(' ')
disp('            STATISTIC RESULTS            ')
disp('             df     T Stat.   S. level   ')
disp('_________________________________________')
disp(['T_r_DSPD      ', num2str(cz,'% 10.0f'), '     ', num2str(T_r_DSPD,'% 10.3f'), '     ', num2str(1-chi2cdf(T_r_DSPD,cz),'% 10.3f')])
disp(['T_rs_DSPD     ', num2str(cz,'% 10.0f'), '     ', num2str(T_rs_DSPD,'% 10.3f'), '     ', num2str(1-chi2cdf(T_rs_DSPD,cz),'% 10.3f')])
