% ************************************************************************ %
%                                                                          %
%                 Diagnostic Tests for Homoskedasticity                    %
%              in Spatial Cross-Sectional or Panel Models                  %
%              Matlab code created by A. Pirotte & Z. Yang                 %
%                                                                          %
%         Spatial Linear Regression (SLR) Model - September 2020           %
%                 (Section 2 of the paper mentioned above)                 %
%                                                                          %
%                          Cigarette demand data                           %
%             of Baltagi and Levin (1986), Baltagi and Li (2004)           %
%                                                                          %
%************************************************************************* %

% No need to modify the environment initialization

clc; clear global; close all;
format short; echo off;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%% PART TO MODIFY %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% You have to change this part following the characteristics of your own
% data set

disp('              Cigarette demand data set                 ') % Should be modified
disp(' of Baltagi and Levin (1986), Baltagi and Li (2004)     ') % Should be modified
disp('   ')                                    

% Data dimension characteristics

n   = 46 ; % States (number of individuals)                                                     % Should be modified
t   = 1  ; % 1963-1992 (number of years, months... considered)                                  % Should be modified
nt  = n*t; % Total number of observations                                                       
tc  = 1  ; % Year retained for Cross-section                                                    % Should be modified
mattime = [(1:t);(1982:1982)]';                                                                 % Should be modified
In      = eye(n)              ;

% Display of the dimensions of the data set and variable names
disp(['Number of US States (N)    ',num2str(n)])                           % Should be modified
disp(['Number of periods (T)      ',num2str(t),'   (1982)'])               % Should be modified
disp(['Number of obs. (NT)        ',num2str(nt)])                          % Should be modified
disp(' ')                                                              
disp('List of the variables ')                                         
init = ['y  = ln(CIG)   - Real per capita sales of cigarettes          ';  % Should be modified
        'x1 = ln(P)     - Average retail price of a pack of cigarettes ';  % Should be modified
        'x2 = ln(I)     - Real per capita disposable Income            ';  % Should be modified
        'x3 = ln(Pop)   - Population                                   ';  % Should be modified  
        'x4 = ln(Pop16) - Population above 16                          ']; % Should be modified
disp(init)

% Spatial matrix - First order contiguity matrix

wb = xlsread('Spat-Sym-US.xls')          ; % uploaded the spatial contiguity U.S. states matrix    % Should be modified
w  = wb./kron(ones(1,n),(ones(1,n)*wb')'); % Row-normalization
ev = eig(w)                              ;

%  Variables 

dcig   = xlsread('cigarette82.xls'); % uploaded the demand data set        % Should be modified
lcig   = dcig(:,3)     ;                                                   % Should be modified                      
lpx    = dcig(:,4)     ;                                                   % Should be modified
ly     = dcig(:,6)     ;                                                   % Should be modified
lpop   = log(dcig(:,7));                                                   % Should be modified
lpop16 = log(dcig(:,8));                                                   % Should be modified
st     = dcig(:,2)     ;                                                   % Should be modified

locig   = reshape(lcig,n,t)  ;                                             % Should be modified
lopx    = reshape(lpx,n,t)   ;                                             % Should be modified
loy     = reshape(ly,n,t)    ;                                             % Should be modified
lopop   = reshape(lpop,n,t)  ;                                             % Should be modified
lopop16 = reshape(lpop16,n,t);                                             % Should be modified
state   = reshape(st,n,t)    ;                                             % Should be modified

% Content identification of Yn and Xn 

Yn        = locig(:,t)                        ;                          % Should be modified
Xn        = [ones(n,1) lopx(:,t) loy(:,t)]    ;                          % Should be modified
[lXM,cXM] = size(Xn)                          ;
dx        = length(Xn(1,:))                   ;

% Content identification of Zn (heteroskedasdicity variables)
Zn        = [lopx(:,t) loy(:,t) lopop16(:,t)];                          % Should be modified
[~,cz]    = size(Zn)                         ; 

disp(' ')
disp('  Spatial Linear Regression (SLR) Model  ')
disp('_________________________________________')
disp(' ')
disp(['Number of obs.    ',num2str(n),'    (Year ',num2str(mattime(1,2)),')'])

disp(' ')
disp('Zn variables : x1 x2 x4')                                            % Should be modified

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%% END OF MODIFICATIONS %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% QML estimator (under H0)

z0    = [0.1  ; 0.2]                 ;
lb    = [-0.99; -0.99]               ;
ub    = [ 0.99;  0.99]               ;
lamh  = Fn_CrossDSL(z0,lb,ub,Xn,Yn,w);
lam1h = lamh(1); lam2h = lamh(2)     ;

B1nh    = In - lam1h*w            ;   
B2nh    = In - lam2h*w            ; 
IB1nh   = pinv(B1nh)              ;
IB2nh   = pinv(B2nh)              ;
YnLh    = B2nh*B1nh*Yn            ;
XnL2h   = B2nh*Xn                 ;
PXnL2h  = pinv(XnL2h'*XnL2h)      ;
MnL2h   = In - XnL2h*PXnL2h*XnL2h';
betah   = (PXnL2h*XnL2h'*YnLh)    ;            
sig2h   = (1/n)*YnLh'*MnL2h*YnLh  ;   
res_QML = YnLh - XnL2h*betah      ;

% Test for homoskedasticity   %
 
G1n  = w*IB1nh                         ;
G2n  = w*IB2nh                         ;
Vnh  = YnLh - XnL2h*betah              ;
PHI1 = (0.5/sig2h^2)*In                ;
PHI2 = B2nh*G1n*IB2nh/sig2h            ;
PHI3 = G2n/sig2h                       ;
Pi1  = XnL2h/sig2h                     ;
Pi2  = B2nh*G1n*IB2nh*XnL2h*betah/sig2h;

xi1   = (triu(PHI1,1)' + tril(PHI1,-1))*Vnh              ;
xi2   = (triu(PHI2,1)' + tril(PHI2,-1))*Vnh              ; 
xi3   = (triu(PHI3,1)' + tril(PHI3,-1))*Vnh              ; 
aaii1 = diag(PHI1)                                       ;
aaii2 = diag(PHI2)                                       ;
aaii3 = diag(PHI3)                                       ;                       
Vnhh  = kron(ones(1,cXM),Vnh)                            ;
g1    = Pi1.*Vnhh                                        ;
g2    = (Vnh.*xi1 + (Vnh.^2 - sig2h).*aaii1)             ;
g3    = (Vnh.*xi2 + (Vnh.^2 - sig2h).*aaii2) + (Vnh.*Pi2);
g4    = (Vnh.*xi3 + (Vnh.^2 - sig2h).*aaii3)             ;
g5    = 0.5*(kron(ones(1,cz),Vnh.^2 - sig2h).*Zn)/sig2h  ; 

% Statistic T_SLR %

g_ni_theta  = [g1 g2 g3 g4]                                        ;
g_ni_alpha  = g5                                                   ;
sg_ni_alpha = sum(g_ni_alpha)                                      ;
K_n_tilde   = (g_ni_alpha'*g_ni_theta)*pinv(g_ni_theta'*g_ni_theta);
MatGG       = g_ni_alpha - g_ni_theta*K_n_tilde'                   ;
T_SLR       = sg_ni_alpha*pinv(MatGG'*MatGG)*sg_ni_alpha'          ;

% Computation Gamma_tilde_n

SigBB       = - (1/sig2h)*(XnL2h'*XnL2h)                              ;
SigBsg      = - (1/(sig2h^2))*XnL2h'*Vnh                              ;
SigBlam1    = - (1/sig2h)*XnL2h'*B2nh*w*Yn                            ;
SigBlam2    = - (1/sig2h)*Xn'*(w'*B2nh + B2nh'*w)*(B1nh*Yn - Xn*betah);

SigsgB      = - (1/(sig2h^2))*XnL2h'*Vnh                  ;
Sigsgsg     = - (1/(sig2h^3))*(Vnh'*Vnh) + (n/(2*sig2h^2));
Sigsglam1   = - (1/(sig2h^2))*Vnh'*B2nh*w*Yn              ;
Sigsglam2   = - (1/(sig2h^2))*Vnh'*G2n*Vnh                ;

Siglam1B    = - (1/(sig2h))*(XnL2h'*B2nh)*w*Yn                             ;
Siglam1sg   = - (1/(sig2h^2))*Vnh'*B2nh*w*Yn                               ;
Siglam1lam1 = - (1/(sig2h))*Yn'*w'*(B2nh'*B2nh)*w*Yn - sum(diag(G1n^2))    ;
Siglam1lam2 = - (1/(sig2h))*Yn'*w'*(B2nh'*w + w'*B2nh)*(B1nh*Yn - Xn*betah);
           
Siglam2B    = - (1/sig2h)*Xn'*(w'*B2nh + B2nh'*w)*(B1nh*Yn - Xn*betah)     ;                                       
Siglam2sg   = - (1/(sig2h^2))*Vnh'*G2n*Vnh                                 ;                                        
Siglam2lam1 = - (1/(sig2h))*Yn'*w'*(B2nh'*w + w'*B2nh)*(B1nh*Yn - Xn*betah);
Siglam2lam2 = - (1/(sig2h))*Vnh'*(G2n'*G2n)*Vnh                            ;   

Sigma_tilde = - [SigBB     SigBsg    SigBlam1    SigBlam2   ;...
                 SigsgB'   Sigsgsg   Sigsglam1   Sigsglam2  ;...
                 Siglam1B' Siglam1sg Siglam1lam1 Siglam1lam2;...
                 Siglam2B' Siglam2sg Siglam2lam1 Siglam2lam2];   

GamalB      = - (1/sig2h)*((kron(ones(1,cz),Vnh).*Zn)'*XnL2h)';
Gamalsg     = - (1/(2*sig2h^2))*diag(Vnh*Vnh')'*Zn            ;
Gamallam1   = - (1/sig2h)*(B2nh*w*Yn.*Vnh)'*Zn                ;
Gamallam2   = - (1/sig2h)*((w*B1nh*Yn - w*Xn*betah).*Vnh)'*Zn ;
Gamma_tilde = - [GamalB ;  Gamalsg ; Gamallam1 ; Gamallam2]'  ;

K_r_tilde   = Gamma_tilde*pinv(Sigma_tilde)                  ;
MatGG       = g_ni_alpha - g_ni_theta*K_r_tilde'             ;
T_r_SLR     = sg_ni_alpha*pinv(MatGG'*MatGG)*sg_ni_alpha'    ; 

%%%%%%  Finite Sample Improved Tests %%%%%%

lamh   = FnSararA(Yn,Xn,w,[lam1h,lam2h]);   % this gives \tilde\lambda^*
B1nh   = In - lamh(1)*w                 ;   
B2nh   = In - lamh(2)*w                 ; 
IB1nh  = pinv(B1nh)                     ;
IB2nh  = pinv(B2nh)                     ;
YnLh   = B2nh*B1nh*Yn                   ;
XnL2h  = B2nh*Xn                        ;
PXnL2h = pinv(XnL2h'*XnL2h)             ;
MnL2h  = In - XnL2h*PXnL2h*XnL2h'       ;
betah  = (PXnL2h*XnL2h'*YnLh)           ;           
sig2h  = (1/n)*YnLh'*MnL2h*YnLh         ;
G1n    = w*IB1nh                        ;
G2n    = w*IB2nh                        ;
Vnh    = YnLh - XnL2h*betah             ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Mn      = MnL2h                                           ;
G1n_bar = (1/n)*trace(G1n)                                ;
G2n_bar = (1/n)*trace(G2n)                                ;
PI_M    = Mn*(B2nh*G1n*IB2nh - G1n_bar.*In)*XnL2h*betah   ; 
PHI_1   = Mn*(B2nh*G1n*IB2nh - G1n_bar.*In)               ; 
PHI_2   = Mn*(G2n - G2n_bar.*In)*Mn                       ; 
imi     = (1./sum(Mn.^2))'                                ;  
sig2b   = (n/(n-cXM))*sig2h                               ;
xi_star = 0.5*(imi.*Vnh.^2 - sig2b.*ones(n,1))            ;
g3      = Zn.*xi_star                                     ;

PHI_1s = PHI_1 - (1/(n-cXM))*trace(PHI_1)*Mn              ;
PHI_2s = PHI_2 - (1/(n-cXM))*trace(PHI_2)*Mn              ;
aaii1s = diag(PHI_1s)                                     ;
aaii2s = diag(PHI_2s)                                     ;
xi1cs  = (triu(PHI_1s,1)' + tril(PHI_1s,-1))*Vnh          ;
xi2cs  = (triu(PHI_2s,1)' + tril(PHI_2s,-1))*Vnh          ;
g1     = Vnh.*xi1cs + (Vnh.^2 - sig2h).*aaii1s + Vnh.*PI_M;    
g2     = Vnh.*xi2cs + (Vnh.^2 - sig2h).*aaii2s            ;   

% Statistic T^*_SLR %

g_ni_lambda = [g1 g2]                                                 ;
g_ni_alpha  = g3                                                      ;
sg_ni_alpha = sum(g_ni_alpha)                                         ;
K_ns_tilde  = (g_ni_alpha'*g_ni_lambda)*pinv(g_ni_lambda'*g_ni_lambda);
MatGG       = g_ni_alpha - g_ni_lambda*K_ns_tilde'                    ;
T_s_SLR     = sg_ni_alpha*pinv(MatGG'*MatGG)*sg_ni_alpha'             ;

% Numerical derivatives %

epse       = 0.00001                                                       ; 
L1v        = lamh(1) + epse                                                ;
L2v        = lamh(2) + epse                                                ; 
B1nhL1v    = In - L1v*w                                                    ;   
B2nhL2v    = In - L2v*w                                                    ; 
IB1nhL1v   = pinv(B1nhL1v)                                                 ;   
IB2nhL2v   = pinv(B2nhL2v)                                                 ; 
YnLhL1v    = B2nh*B1nhL1v*Yn                                               ;
YnLhL2v    = B2nhL2v*B1nh*Yn                                               ;
MnL2v      = In - B2nhL2v*Xn*pinv(Xn'*(B2nhL2v)'*B2nhL2v*Xn)*Xn'*B2nhL2v'  ; 
sig2hL1v   = (1/n)*YnLhL1v'*Mn*YnLhL1v                                     ;
sig2hL2v   = (1/n)*YnLhL2v'*MnL2v*YnLhL2v                                  ;
G1nL1v     = w*pinv(B1nhL1v)                                               ;
G2nL2v     = w*pinv(B2nhL2v)                                               ;
G1n_barL1v = (1/n)*trace(G1nL1v)                                           ;
G2n_barL2v = (1/n)*trace(G2nL2v)                                           ;
PHI_1L1v   = Mn*(B2nh*G1nL1v*IB2nh - G1n_barL1v.*In)                       ;
PHI_1L2v   = MnL2v*(B2nhL2v*G1n*IB2nhL2v - G1n_bar.*In)                    ;  
PHI_2L1v   = Mn*(G2n - G2n_bar.*In)*Mn                                     ; 
PHI_2L2v   = MnL2v*(G2nL2v - G2n_barL2v.*In)*MnL2v                         ; 

SsSACPHI1    = YnLh'*PHI_1*YnLh - (n/(n-cXM))*sig2h*trace(PHI_1)               ; 
SsSACPHI2    = YnLh'*PHI_2*YnLh - (n/(n-cXM))*sig2h*trace(PHI_2)               ;
SsSACPHI1L1v = YnLhL1v'*PHI_1L1v*YnLhL1v - (n/(n-cXM))*sig2hL1v*trace(PHI_1L1v); 
SsSACPHI1L2v = YnLhL2v'*PHI_1L2v*YnLhL2v - (n/(n-cXM))*sig2hL2v*trace(PHI_1L2v); 
SsSACPHI2L1v = YnLhL1v'*PHI_2L1v*YnLhL1v - (n/(n-cXM))*sig2hL1v*trace(PHI_2L1v);
SsSACPHI2L2v = YnLhL2v'*PHI_2L2v*YnLhL2v - (n/(n-cXM))*sig2hL2v*trace(PHI_2L2v);

Sigl1l1          = (SsSACPHI1L1v - SsSACPHI1)/epse     ;
Sigl1l2          = (SsSACPHI1L2v - SsSACPHI1)/epse     ;
Sigl2l1          = (SsSACPHI2L1v - SsSACPHI2)/epse     ;
Sigl2l2          = (SsSACPHI2L2v - SsSACPHI2)/epse     ;
Sigma_s_tildell  = -[Sigl1l1 Sigl1l2 ; Sigl2l1 Sigl2l2];

SsSAC    = 0.5*Zn'*(imi.*Vnh.^2 - (1/(n-cXM))*Yn'*B1nh'*B2nh'*Mn*B2nh*B1nh*Yn.*ones(n,1));
imiL2v   = (1./sum(MnL2v.^2))'                                                           ;
VnhL1v   = Mn*B2nh*B1nhL1v*Yn                                                            ;
VnhL2v   = MnL2v*B2nhL2v*B1nh*Yn                                                         ;
termL1v  = B1nhL1v'*B2nh'*Mn*B2nh*B1nhL1v                                                ;
termL2v  = B2nhL2v'*MnL2v*B2nhL2v                                                        ;
SsSACL1v = 0.5*Zn'*(imi.*VnhL1v.^2    - (1/(n-cXM))*Yn'*termL1v*Yn.*ones(n,1))           ;
SsSACL2v = 0.5*Zn'*(imiL2v.*VnhL2v.^2 - (1/(n-cXM))*Yn'*B1nh'*termL2v*B1nh*Yn.*ones(n,1));
Gammall1 = (SsSACL1v - SsSAC)/epse                                                       ;  
Gammall2 = (SsSACL2v - SsSAC)/epse                                                       ; 
                  
Sigma_s_tildeal = - [Gammall1 Gammall2]                      ;
K_rs_tilde      = Sigma_s_tildeal*pinv(Sigma_s_tildell)      ;
MatGG           = g_ni_alpha - g_ni_lambda*K_rs_tilde'       ;
T_rs_SLR        = sg_ni_alpha*pinv(MatGG'*MatGG)*sg_ni_alpha';

disp(' ')
disp('            STATISTIC RESULTS            ')
disp('             df     T Stat.   S. level   ')
disp('_________________________________________')
disp(['T_SLR         ', num2str(cz,'% 10.0f'), '     ', num2str(T_SLR,'% 10.3f'), '     ', num2str(1-chi2cdf(T_SLR,cz),'% 10.3f')])
disp(['T_r_SLR       ', num2str(cz,'% 10.0f'), '     ', num2str(T_r_SLR,'% 10.3f'), '     ', num2str(1-chi2cdf(T_r_SLR,cz),'% 10.3f')])
disp(['T_s_SLR       ', num2str(cz,'% 10.0f'), '     ', num2str(T_s_SLR,'% 10.3f'), '     ', num2str(1-chi2cdf(T_s_SLR,cz),'% 10.3f')])
disp(['T_rs_SLR      ', num2str(cz,'% 10.0f'), '     ', num2str(T_rs_SLR,'% 10.3f'), '     ', num2str(1-chi2cdf(T_rs_SLR,cz),'% 10.3f')]) 
disp('    ')