% ************************************************************************ %
%                                                                          %
%                 Diagnostic Tests for Homoskedasticity                    %
%              in Spatial Cross-Sectional or Panel Models                  %
%              Matlab code created by A. Pirotte & Z. Yang                 %
%                                                                          %
%            Spatial Panel Data (SPD) Model - September 2020               %
%                 (Section 3 of the paper mentioned above)                 %
%                                                                          %
%                          Cigarette demand data                           %
%             of Baltagi and Levin (1986), Baltagi and Li (2004)           %
%                                                                          %
%************************************************************************* %

% No need to modify the environment initialization

clc; clear global; close all;
format short; echo off;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%% PART TO MODIFY %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% You have to change this part following the characteristics of your own
% data set

disp('              Cigarette demand data set                 ')           % Should be modified
disp(' of Baltagi and Levin (1986), Baltagi and Li (2004)     ')           % Should be modified
disp('   ')

% Data dimension characteristics

n       = 46                ; % States (number of individuals)             % Should be modified
t       = 5                 ; % 1980-1984 (number of years, months...)     % Should be modified
nt      = n*t               ; % Total number of observations                               
In    = eye(n)              ;
It    = eye(t)              ;
It_1  = eye(t-1)            ;
e_n   = ones(n,1)           ;
e_T_1 = ones(t-1,1)         ;

% Display of the dimensions of the data set and variable names
disp(['Number of US States (N)    ',num2str(n)])                           % Should be modified
disp(['Number of periods (T)      ',num2str(t),'   (1980-1984)'])          % Should be modified
disp(['Number of obs. (NT)        ',num2str(nt)])                          % Should be modified
disp(' ')                                                              
disp('List of the variables ')                                         
init = ['y  = ln(CIG)   - Real per capita sales of cigarettes          ';  % Should be modified
        'x1 = ln(P)     - Average retail price of a pack of cigarettes ';  % Should be modified
        'x2 = ln(I)     - Real per capita disposable Income            ';  % Should be modified
        'x3 = ln(Pop)   - Population                                   ';  % Should be modified  
        'x4 = ln(Pop16) - Population above 16                          ']; % Should be modified
disp(init)

% Spatial matrix - First order contiguity matrix

wb = xlsread('Spat-Sym-US.xls')          ; % Uploaded the spatial contiguity U.S. states matrix    % Should be modified
w  = wb./kron(ones(1,n),(ones(1,n)*wb')'); % Row-normalization
ev = eig(w)                              ;
lb = 1/min(ev) + 0.0001                  ;
ub = 1/max(ev) - 0.0001                  ;
w1 = kron(It_1,w)                        ;

%  Variables 

dcig   = xlsread('cigarette80_84.xls'); % uploaded the demand data set     % Should be modified
lcig   = dcig(:,3)     ;                                                   % Should be modified                      
lpx    = dcig(:,4)     ;                                                   % Should be modified
ly     = dcig(:,6)     ;                                                   % Should be modified
lpop   = log(dcig(:,7));                                                   % Should be modified
lpop16 = log(dcig(:,8));                                                   % Should be modified
st     = dcig(:,2)     ;                                                   % Should be modified

locig   = reshape(lcig,n,t)  ;                                             % Should be modified
lopx    = reshape(lpx,n,t)   ;                                             % Should be modified
loy     = reshape(ly,n,t)    ;                                             % Should be modified
lopop   = reshape(lpop,n,t)  ;                                             % Should be modified
lopop16 = reshape(lpop16,n,t);                                             % Should be modified
state   = reshape(st,n,t)    ;                                             % Should be modified

lcig   = reshape(locig(:,1:t),nt,1)  ;                                     % Should be modified
lpx    = reshape(lopx(:,1:t),nt,1)   ;                                     % Should be modified
ly     = reshape(loy(:,1:t),nt,1)    ;                                     % Should be modified 
lpop16 = reshape(lopop16(:,1:t),nt,1);                                     % Should be modified 

% Transformation approach by Lee and Yu (2010), Yand et al. (2016) - F(T,T-1)

[V,D]  = eig(It - (1/t).*ones(t))        ;
D      = diag(D)                         ;           
j      = find(D <= 0.0001 & D >= -0.0001);
V(:,j) = []                              ;
F      = V                               ;
TR_Y   = kron(F',In)                     ;

% Individual means of Zn variables (Heteroskedascity variables)

lpx     = TR_Y*lpx                     ;                                     % Should be modified
ly      = TR_Y*ly                      ;                                     % Should be modified
lpop16  = TR_Y*lpop16                  ;                                     % Should be modified
i_XN1   = mean(reshape(lpx,n,t-1),2)   ; % vector of individual means of x1  % Should be modified
i_XN2   = mean(reshape(ly,n,t-1),2)    ; % vector of individual means of x2  % Should be modified
i_XN3   = mean(reshape(lpop16,n,t-1),2); % vector of individual means of x4  % Should be modified
Z1      = kron(ones(t-1,1),i_XN1)      ;                                     % Should be modified
Z2      = kron(ones(t-1,1),i_XN2)      ;                                     % Should be modified
Z3      = kron(ones(t-1,1),i_XN3)      ;                                     % Should be modified
ZN      = [Z1 Z2 Z3]                   ;                                     % Should be modified
[lz,cz] = size(ZN)                     ;

YN  = TR_Y*lcig      ;                                                       % Should be modified
XN  = [lpx ly]       ;                                                       % Should be modified
dx  = length(XN(1,:));
X_M = XN             ;

disp(' ')
disp('    Spatial Panel Data (SPD) Model    ')
disp('______________________________________')
disp(' ')
disp('Individual means of Zn variables : x1(i.) x2(i.) x4(i.)')              % Should be modified

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%% END OF MODIFICATIONS %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% QML estimator (under H0)  

[lamh] = FEnSarar(YN,XN,w,ev,n,lb,ub);
lam1h  = lamh(1); lam2h = lamh(2);   

B1nh     = In - lam1h*w              ;   
B2nh     = In - lam2h*w              ; 
IB1nh    = pinv(B1nh)                ;
IB2nh    = pinv(B2nh)                ;
It1n     = kron(It_1,In)             ;
B1Nh     = kron(It_1,B1nh)           ;   
B2Nh     = kron(It_1,B2nh)           ; 
IB2Nh    = kron(It_1,IB2nh)          ;
YNLh     = B2Nh*B1Nh*YN              ;
XNL2h    = B2Nh*XN                   ;
PROXNL2h = (XNL2h)'*XNL2h            ;
PXNL2h   = pinv(PROXNL2h)            ;
MNL2h    = It1n - XNL2h*PXNL2h*XNL2h';
betah    = (PXNL2h*XNL2h'*YNLh)      ;           
sig2h    = (1/lz)*YNLh'*MNL2h*YNLh   ;   

% Test for homoskedasticity   %

G1N      = kron(It_1,w*IB1nh)     ;
G2N      = kron(It_1,w*IB2nh)     ;
iSig_2_v = 1/sig2h                ;
VNh      = YNLh - XNL2h*betah     ;
PHI1     = 0.5*(iSig_2_v^2)*It1n  ;
PHI2     = iSig_2_v*B2Nh*G1N*IB2Nh;
PHI3     = iSig_2_v*G2N           ;
Pi1      = iSig_2_v*XNL2h         ;
Pi2      = PHI2*XNL2h*betah       ;

xi1      = (triu(PHI1,1)' + tril(PHI1,-1))*VNh      ; 
xi2      = (triu(PHI2,1)' + tril(PHI2,-1))*VNh      ; 
xi3      = (triu(PHI3,1)' + tril(PHI3,-1))*VNh      ; 

aaii1    = diag(PHI1)                               ;
aaii2    = diag(PHI2)                               ;
aaii3    = diag(PHI3)                               ;                       
VNhh     = kron(ones(1,dx),VNh)                     ;
dVN_Sig2 = (VNh.^2 - sig2h)                         ;
g1       = Pi1.*VNhh                                ;
g2       = (VNh.*xi1 + dVN_Sig2.*aaii1)             ;
g3       = (VNh.*xi2 + dVN_Sig2.*aaii2) + (VNh.*Pi2);
g4       = (VNh.*xi3 + dVN_Sig2.*aaii3)             ;
g5       = 0.5*(kron(ones(1,cz),dVN_Sig2).*ZN)/sig2h;

% Statistic T_SPD %

g_ni_theta  = [g1 g2 g3 g4]                                        ;
g_ni_alpha  = g5                                                   ;
sg_ni_alpha = sum(g_ni_alpha)                                      ;
K_tilde_N   = (g_ni_alpha'*g_ni_theta)*pinv(g_ni_theta'*g_ni_theta);
MatGG       = g_ni_alpha - g_ni_theta*K_tilde_N'                   ;
T_SPD       = sg_ni_alpha*pinv(MatGG'*MatGG)*sg_ni_alpha'          ;       

% Statistic T^r_SPD %

Terme1      = (w1'*B2Nh + B2Nh'*w1);
Terme2      = (B1Nh*YN - X_M*betah);

SigBB       = - iSig_2_v*(PROXNL2h)        ;       
SigBsg      = - (iSig_2_v^2)*XNL2h'*VNh    ;
SigBlam1    = - iSig_2_v*XNL2h'*B2Nh*w1*YN ;
SigBlam2    = - iSig_2_v*X_M'*Terme1*Terme2;

SigsgB      = - (iSig_2_v^2)*XNL2h'*VNh                   ;
Sigsgsg     = - (iSig_2_v^3)*(VNh'*VNh) + (lz/(2*sig2h^2));
Sigsglam1   = - (iSig_2_v^2)*VNh'*B2Nh*w1*YN              ;
Sigsglam2   = - (iSig_2_v^2)*VNh'*G2N*VNh                 ;

Siglam1B    = - iSig_2_v*(XNL2h'*B2Nh)*w1*YN                          ; 
Siglam1sg   = - (iSig_2_v^2)*VNh'*B2Nh*w1*YN                          ;
Siglam1lam1 = - iSig_2_v*YN'*w1'*(B2Nh'*B2Nh)*w1*YN - sum(diag(G1N^2));
Siglam1lam2 = - iSig_2_v*YN'*w1'*Terme1'*Terme2                       ;

Siglam2B    = - iSig_2_v*X_M'*Terme1*Terme2    ;                                       
Siglam2sg   = - (iSig_2_v^2)*VNh'*G2N*VNh      ;                                        
Siglam2lam1 = - iSig_2_v*YN'*w1'*Terme1'*Terme2;
Siglam2lam2 = - iSig_2_v*VNh'*(G2N'*G2N)*VNh   ;   

Sigma_tilde = - [SigBB     SigBsg    SigBlam1    SigBlam2   ;...
                 SigsgB'   Sigsgsg   Sigsglam1   Sigsglam2  ;...
                 Siglam1B' Siglam1sg Siglam1lam1 Siglam1lam2;...
                 Siglam2B' Siglam2sg Siglam2lam1 Siglam2lam2];   

GamalB      = - iSig_2_v*((kron(ones(1,cz),VNh).*ZN)'*XNL2h)';
Gamalsg     = - 0.5*(iSig_2_v^2)*diag(VNh*VNh')'*ZN          ;
Gamallam1   = - iSig_2_v*(B2Nh*w1*YN.*VNh)'*ZN               ;
Gamallam2   = - iSig_2_v*((w1*Terme2).*VNh)'*ZN              ;
Gamma_tilde = - [GamalB ; Gamalsg; Gamallam1; Gamallam2]'    ;

Gamma_tilde_N = Gamma_tilde*pinv(Sigma_tilde)                ;
MatGG         = g_ni_alpha - g_ni_theta*Gamma_tilde_N'       ;
T_r_SPD       = sg_ni_alpha*pinv(MatGG'*MatGG)*sg_ni_alpha'  ;

% Statistic T^rr_SPD % 

d1_N = Pi1.*VNhh                  ;
d2_N = dVN_Sig2.*aaii1            ;
d3_N = dVN_Sig2.*aaii2 + VNh.*Pi2 ;
d4_N = dVN_Sig2.*aaii3            ;
d5_N = 0.5*iSig_2_v*(dVN_Sig2.*ZN);

rd1 = cell(n,t-1);
for i = 1:dx 
  rd1{i} = reshape(d1_N(:,i),n,t-1);
end
rd2  = reshape(d2_N,n,t-1)     ; 
rd3  = reshape(d3_N,n,t-1)     ; 
rd4  = reshape(d4_N,n,t-1)     ; 
rd5 = cell(n,t-1);
for i = 1:cz  
  rd5{i} = reshape(d5_N(:,i),n,t-1);
end

d1_i = cell(n*(t-1),1);
for i = 1:dx  
  d1_i{i} = reshape(rd1{i}',n*(t-1),1); % Stacking by individual
end
d2_i  = reshape(rd2',n*(t-1),1) ;       % Stacking by individual
d3_i  = reshape(rd3',n*(t-1),1) ;       % Stacking by individual
d4_i  = reshape(rd4',n*(t-1),1) ;       % Stacking by individual
d5_i = cell(n*(t-1),1);
for i = 1:cz  
  d5_i{i} = reshape(rd5{i}',n*(t-1),1); % Stacking by individual
end

M_en1 = kron(e_n,(e_T_1*e_T_1' - It_1));
rd1s  = cell(n*(t-1),t-1);
for i = 1:dx  
  rd1s{i} = (kron(rd1{i},e_T_1).*M_en1)';
end
rd2s    = (kron(rd2,e_T_1).*M_en1)'     ;
rd3s    = (kron(rd3,e_T_1).*M_en1)'     ;
rd4s    = (kron(rd4,e_T_1).*M_en1)'     ;
rd5s = cell(n*(t-1),t-1);
for i = 1:cz  
  rd5s{i} = (kron(rd5{i},e_T_1).*M_en1)';
end

d1_o_i  = cell(t-1,1);
for i = 1:dx  
  d1_o_i{i} = sum(rd1s{i})';
end
d2_o_i  = sum(rd2s)'                     ;    
d3_o_i  = sum(rd3s)'                     ;
d4_o_i  = sum(rd4s)'                     ;
d5_o_i  = cell(t-1,1);
for i = 1:cz  
  d5_o_i{i} = sum(rd5s{i})';
end

mat1   = [];
mat1_o = [];
for i = 1:dx
  mat1   = [mat1   d1_i{i}  ];
  mat1_o = [mat1_o d1_o_i{i}];
end
mat5   = [];
mat5_o = [];
for i = 1:cz
  mat5   = [mat5   d5_i{i}  ];
  mat5_o = [mat5_o d5_o_i{i}];
end

d_N_it   = [mat1   d2_i   d3_i   d4_i   mat5  ];
d_N_o_it = [mat1_o d2_o_i d3_o_i d4_o_i mat5_o];

n_P  = cz + dx + 3;
d_P  = zeros(n_P,n_P);
for j = 1:(n*(t-1))
d_P = d_P + d_N_it(j,:)'*d_N_o_it(j,:);        
end
g_tilde_N  = [g_ni_theta g_ni_alpha]     ;
Om_tilde_N = (g_tilde_N'*g_tilde_N + d_P);

T_rr_SPD = sg_ni_alpha*pinv(Om_tilde_N(dx+4:n_P,dx+4:n_P) - 2*Om_tilde_N(dx+4:n_P,1:dx+3)*Gamma_tilde_N' + Gamma_tilde_N*Om_tilde_N(1:dx+3,1:dx+3)*Gamma_tilde_N')*sg_ni_alpha';  

%%%%%%  Finite Sample Improved Tests %%%%%%

lamh  = FEnSararA(YN,XN,w,t,[lam1h,lam2h]);   
lam1h = lamh(1); lam2h = lamh(2);
 
B1nh     = In - lam1h*w              ;
B2nh     = In - lam2h*w              ;
IB1nh    = pinv(B1nh)                ;
IB2nh    = pinv(B2nh)                ;
It1n     = kron(It_1,In)             ;
B1Nh     = kron(It_1,B1nh)           ;
B2Nh     = kron(It_1,B2nh)           ;
IB1Nh    = kron(It_1,IB1nh)          ;
IB2Nh    = kron(It_1,IB2nh)          ;
YNLh     = B2Nh*B1Nh*YN              ;
XNL2h    = B2Nh*XN                   ;
PROXNL2h = (XNL2h)'*XNL2h            ;
PXNL2h   = pinv(PROXNL2h)            ;
MNL2h    = It1n - XNL2h*PXNL2h*XNL2h';
betah    = (PXNL2h*XNL2h'*YNLh)      ;
sig2h    = (1/lz)*YNLh'*MNL2h*YNLh   ;
G1N      = kron(It_1,w*IB1nh)        ;
G2N      = kron(It_1,w*IB2nh)        ;
VNh      = YNLh - XNL2h*betah        ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

MN      = MNL2h                                               ;
G1N_bar = (1/lz)*trace(G1N)                                   ;
G2N_bar = (1/lz)*trace(G2N)                                   ;
PHI_1   = MN*(B2Nh*G1N*IB2Nh - G1N_bar.*It1n)                 ; 
PI_M    = PHI_1*XNL2h*betah                                   ;  
PHI_2   = MN*(G2N - G2N_bar.*eye(lz))*MN                      ; 
imi     = (1./sum(MN.^2))'                                    ;
sig2b   = (lz/(lz-dx))*sig2h                                  ;
xi_star = 0.5*(imi.*VNh.^2 - sig2b)                           ;
g3      = (ZN.*xi_star)                                       ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PHI_1s = PHI_1 - (1/(n-dx))*trace(PHI_1)*MN               ;
PHI_2s = PHI_2 - (1/(n-dx))*trace(PHI_2)*MN               ;
aaii1s = diag(PHI_1s)                                     ;
aaii2s = diag(PHI_2s)                                     ;
xi1cs  = (triu(PHI_1s,1)' + tril(PHI_1s,-1))*VNh          ;
xi2cs  = (triu(PHI_2s,1)' + tril(PHI_2s,-1))*VNh          ;
g1     = VNh.*xi1cs + (VNh.^2 - sig2h).*aaii1s + VNh.*PI_M;
g2     = VNh.*xi2cs + (VNh.^2 - sig2h).*aaii2s            ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Statistic T^*_SPD %

g_ni_lambda   = [g1 g2]                                                 ;
g_ni_alpha    = g3                                                      ;
sg_ni_alpha   = sum(g_ni_alpha)                                         ;
Gamma_tilde_s = (g_ni_alpha'*g_ni_lambda)*pinv(g_ni_lambda'*g_ni_lambda);
MatGG         = g_ni_alpha - g_ni_lambda*Gamma_tilde_s'                 ;
T_s_SPD     = sg_ni_alpha*pinv(MatGG'*MatGG)*sg_ni_alpha'               ;

% Numerical derivatives %

epse       = 0.00001                                                           ;  % Numerical derivatives
L1v        = lamh(1) + epse                                                    ;
L2v        = lamh(2) + epse                                                    ;
B1nhL1v    = In - L1v*w                                                        ;   
B2nhL2v    = In - L2v*w                                                        ; 
B1NhL1v    = kron(It_1,B1nhL1v)                                                ;  
B2NhL2v    = kron(It_1,B2nhL2v)                                                ; 
IB1NhL1v   = pinv(B1NhL1v)                                                     ;   
IB2NhL2v   = pinv(B2NhL2v)                                                     ; 
YNLhL1v    = B2Nh*B1NhL1v*YN                                                   ;
YNLhL2v    = B2NhL2v*B1Nh*YN                                                   ;
MNL2v      = It1n - B2NhL2v*X_M*pinv(X_M'*(B2NhL2v)'*B2NhL2v*X_M)*X_M'*B2NhL2v'; 
sig2hL1v   = (1/lz)*YNLhL1v'*MN*YNLhL1v                                        ;
sig2hL2v   = (1/lz)*YNLhL2v'*MNL2v*YNLhL2v                                     ;
G1NL1v     = kron(It_1,w)*pinv(B1NhL1v)                                        ;
G2NL2v     = kron(It_1,w)*pinv(B2NhL2v)                                        ;
G1N_barL1v = (1/lz)*trace(G1NL1v)                                              ;
G2N_barL2v = (1/lz)*trace(G2NL2v)                                              ;
PHI_1L1v   = MN*(B2Nh*G1NL1v*IB2Nh - G1N_barL1v.*It1n)                         ;
PHI_1L2v   = MNL2v*(B2NhL2v*G1N*IB2NhL2v - G1N_bar.*It1n)                      ;  
PHI_2L1v   = MN*(G2N - G2N_bar.*It1n)*MN                                       ;  
PHI_2L2v   = MNL2v*(G2NL2v - G2N_barL2v.*It1n)*MNL2v                           ; 

SsFESACPHI1    = YNLh'*PHI_1*YNLh - (lz/(lz-dx))*sig2h*trace(PHI_1)               ; 
SsFESACPHI2    = YNLh'*PHI_2*YNLh - (lz/(lz-dx))*sig2h*trace(PHI_2)               ;
SsFESACPHI1L1v = YNLhL1v'*PHI_1L1v*YNLhL1v - (lz/(lz-dx))*sig2hL1v*trace(PHI_1L1v); 
SsFESACPHI1L2v = YNLhL2v'*PHI_1L2v*YNLhL2v - (lz/(lz-dx))*sig2hL2v*trace(PHI_1L2v); 
SsFESACPHI2L1v = YNLhL1v'*PHI_2L1v*YNLhL1v - (lz/(lz-dx))*sig2hL1v*trace(PHI_2L1v);
SsFESACPHI2L2v = YNLhL2v'*PHI_2L2v*YNLhL2v - (lz/(lz-dx))*sig2hL2v*trace(PHI_2L2v);

Sigl1l1       = (SsFESACPHI1L1v - SsFESACPHI1)/epse ;
Sigl1l2       = (SsFESACPHI1L2v - SsFESACPHI1)/epse ;
Sigl2l1       = (SsFESACPHI2L1v - SsFESACPHI2)/epse ;
Sigl2l2       = (SsFESACPHI2L2v - SsFESACPHI2)/epse ;
Sigma_s_tilde = -[Sigl1l1 Sigl1l2 ; Sigl2l1 Sigl2l2];

SsFESAC    = 0.5*ZN'*(imi.*VNh.^2 - (1/(lz-dx))*YN'*B1Nh'*B2Nh'*MN*B2Nh*B1Nh*YN.*ones(lz,1));
imiL2v     = (1./sum(MNL2v.^2))'                                                            ;
VNhL1v     = MN*B2Nh*B1NhL1v*YN                                                             ;
VNhL2v     = MNL2v*B2NhL2v*B1Nh*YN                                                          ;
termL1v    = B1NhL1v'*B2Nh'*MN*B2Nh*B1NhL1v                                                 ;
termL2v    = B2NhL2v'*MNL2v*B2NhL2v                                                         ;
SsFESACL1v = 0.5*ZN'*(imi.*VNhL1v.^2    - (1/(lz-dx))*YN'*termL1v*YN.*ones(lz,1))           ;
SsFESACL2v = 0.5*ZN'*(imiL2v.*VNhL2v.^2 - (1/(lz-dx))*YN'*B1Nh'*termL2v*B1Nh*YN.*ones(lz,1));
Gammall1   = (SsFESACL1v - SsFESAC)/epse                                                    ;  
Gammall2   = (SsFESACL2v - SsFESAC)/epse                                                    ; 

Gamma_s_tilde = - [Gammall1 Gammall2]                      ;
K_rs_tilde    = Gamma_s_tilde*pinv(Sigma_s_tilde)          ;
MatGG         = g_ni_alpha - g_ni_lambda*K_rs_tilde'       ;
T_rs_SPD      = sg_ni_alpha*pinv(MatGG'*MatGG)*sg_ni_alpha';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

d1_N = VNh.*(diag(diag(PHI_1))*VNh) + VNh.*PI_M;
d2_N = VNh.*(diag(diag(PHI_2))*VNh)            ;
d3_N = g3                                      ;

rd1  = reshape(d1_N,n,t-1) ; 
rd2  = reshape(d2_N,n,t-1) ; 
rd3  = cell(n,t-1)         ;
for i = 1:cz  
  rd3{i} = reshape(d3_N(:,i),n,t-1);
end

d1_i  = reshape(rd1',n*(t-1),1) ;      % Stacking by individual
d2_i  = reshape(rd2',n*(t-1),1) ;      % Stacking by individual
d3_i  = cell(n*(t-1),1)         ;
for i = 1:cz  
  d3_i{i} = reshape(rd3{i},n*(t-1),1); % Stacking by individual
end

M_en1   = kron(e_n,(e_T_1*e_T_1' - It_1));
rd1s    = (kron(rd1,e_T_1).*M_en1)'      ;
rd2s    = (kron(rd2,e_T_1).*M_en1)'      ;
rd3s = cell(n*(t-1),t-1);
for i = 1:cz  
  rd3s{i} = (kron(rd3{i},e_T_1).*M_en1)';
end

d1_o_i  = sum(rd1s)'                     ;
d2_o_i  = sum(rd2s)'                     ;
d3_o_i  = cell(t-1,1);
for i = 1:cz  
  d3_o_i{i} = sum(rd3s{i})';
end

mat3   = [];
mat3_o = [];
for i = 1:cz
  mat3   = [mat3   d3_i{i}  ];
  mat3_o = [mat3_o d3_o_i{i}];
end

d_N_it     = [d1_i    d2_i    mat3  ];
d_N_o_it   = [d1_o_i  d2_o_i  mat3_o];

n_P = cz + 2;
d_P = zeros(n_P,n_P);
for j = 1:(n*(t-1))
d_P = d_P + d_N_it(j,:)'*d_N_o_it(j,:);        
end
g_tilde_N  = [g_ni_lambda g_ni_alpha]    ;
Om_tilde_N = (g_tilde_N'*g_tilde_N + d_P);

T_rrs_SPD = sg_ni_alpha*pinv(Om_tilde_N(3:n_P,3:n_P) - 2*Om_tilde_N(3:n_P,1:2)*K_rs_tilde' + K_rs_tilde*Om_tilde_N(1:2,1:2)*K_rs_tilde')*sg_ni_alpha';

disp(' ')
disp('            STATISTIC RESULTS            ')
disp('             df     T Stat.   S. level   ')
disp('_________________________________________')
disp(['T_SPD         ', num2str(cz,'% 10.0f'), '     ', num2str(T_SPD,'% 10.3f'), '     ', num2str(1-chi2cdf(T_SPD,cz),'% 10.3f')])
disp(['T_r_SPD       ', num2str(cz,'% 10.0f'), '     ', num2str(T_r_SPD,'% 10.3f'), '     ', num2str(1-chi2cdf(T_r_SPD,cz),'% 10.3f')])
disp(['T_rr_SPD      ', num2str(cz,'% 10.0f'), '     ', num2str(T_rr_SPD,'% 10.3f'), '     ', num2str(1-chi2cdf(T_rr_SPD,cz),'% 10.3f')])
disp(['T_s_SPD       ', num2str(cz,'% 10.0f'), '     ', num2str(T_s_SPD,'% 10.3f'), '     ', num2str(1-chi2cdf(T_s_SPD,cz),'% 10.3f')])
disp(['T_rs_SPD      ', num2str(cz,'% 10.0f'), '     ', num2str(T_rs_SPD,'% 10.3f'), '     ', num2str(1-chi2cdf(T_rs_SPD,cz),'% 10.3f')])
disp(['T_rrs_SPD     ', num2str(cz,'% 10.0f'), '     ', num2str(T_rrs_SPD,'% 10.3f'), '     ', num2str(1-chi2cdf(T_rrs_SPD,cz),'% 10.3f')])
